﻿/*
Ludum Dare Scratch Program - Application class
*/

#include <windows.h>
#include <d3d9.h>
#pragma hdrstop

#include "tlib.h"
#include "LithGUI.h"
#include "LithDX.h"
#include "LithUtil.h"

#include "CApplication.h"
#include "CMainFormHandler.h"

using namespace tlib;
using namespace LithGUI;
using namespace LithDX;
using namespace LithUtil;



CApplication::CApplication(HINSTANCE aInstance)
{
	mGUI = new CGUI(aInstance);
	mDX = new CDX();

	mDevice = NULL;
}



CApplication::~CApplication()
{
	if (mDevice != NULL)
		delete mDevice;

	delete mDX;
	delete mGUI;
}



void CApplication::Run()
{
	Initialise();

	if (SetTimer(mMainForm->GetWindow(), 1, 20, NULL) == 0)
		throw TException(L"SetTimer() failed");
	MSG Message;
	while (true)
	{
		int Result = GetMessage(&Message, NULL, 0, 0);
		if (Result == -1)
			throw TException(L"GetMessage() failed");
		if (Result == 0)
			break;
		TranslateMessage(&Message);
		DispatchMessage(&Message);
	}
}



void CApplication::Initialise()
{
	mMainForm = mGUI->CreateForm(new CMainFormHandler(this), SL("Ludum Dare Scratch Program"), 800, 600);
	mMainForm->SetVisible(true);

	mDevice = mDX->CreateDevice(dmWindow, mMainForm->GetWindow(), 1024, 768);
	//mDevice->SetBackgroundColour(D3DCOLOR_XRGB(122, 164, 255));
	mDevice->GetDevice()->SetRenderState(D3DRS_LIGHTING, false);
	mDevice->GetDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
	mDevice->GetDevice()->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	mDevice->GetDevice()->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
}



void CApplication::OnDestroy()
{
	PostQuitMessage(0);
	if (KillTimer(mMainForm->GetWindow(), 1) == 0)
		throw TException(L"KillTimer() failed");
}



void CApplication::OnTimer()
{
	// TODO: game logic goes here

	mDevice->BeginFrame();
	// TODO: rendering goes here
	mDevice->EndFrame();
}



void CApplication::OnKeyDown(int aKey)
{
	switch (aKey)
	{
	case VK_ESCAPE:
		if (!PostMessage(mMainForm->GetWindow(), WM_CLOSE, 0, 0))
			throw TException(L"PostMessage() failed");
		break;
	}
}



void CApplication::OnKeyUp(int aKey)
{
}



void CApplication::OnChar(int aChar)
{
}



void CApplication::OnMouseMove(int aX, int aY)
{
}



void CApplication::OnMouse(bool aRight, bool aUp, int aX, int aY)
{
}



CTexture* CApplication::GetTexture(TString aTextureName)
{
	if (!mTextures.ContainsKey(aTextureName))
	{
		TString FileName = aTextureName + SL(".bmp");
		CBitmap *Bitmap = new CBitmap(SL("Data\\") + FileName);
		CTexture *Texture = mDevice->CreateTexture(Bitmap->GetXSize(), Bitmap->GetYSize(),
				Bitmap->GetData());
		delete Bitmap;
		mTextures.Add(aTextureName, Texture);
		return Texture;
	}
	return mTextures[aTextureName];
}



CSprite* CApplication::GetSprite(TString aSpriteName)
{
	if (!mSprites.ContainsKey(aSpriteName))
	{
		TString FileName = aSpriteName + SL(".bmp");
		CBitmap *Bitmap = new CBitmap(SL("Data\\") + FileName);
		CSprite *Sprite = mDevice->CreateSprite(Bitmap->GetXSize(), Bitmap->GetYSize(),
				Bitmap->GetData());
		delete Bitmap;
		mSprites.Add(aSpriteName, Sprite);
		return Sprite;
	}
	return mSprites[aSpriteName];
}